@@ -1,4 +1,4 @@
-# $Id: Repository.pm,v 1.12 2012/10/25 21:10:43 pfeiffer Exp $
+# $Id: Repository.pm,v 1.16 2013/08/19 06:38:27 pfeiffer Exp $
 
 =head1 NAME
 
@@ -11,7 +11,10 @@ The actual functionality is also dispersed in makepp and other modules.
 
 =cut
 
-package Mpp::Repository;	# For CPAN scanner.
+package Mpp::Repository;
+
+our $hits = 0;	  		# Number of the files changed that were imported from a rep.
+
 package Mpp::File;		# Use this a lot.
 
 use Mpp::FileOpt;
@@ -125,6 +128,7 @@ sub Mpp::Repository::load {
   my( $dirinfo, $destdirinfo, $prune_code ) = @_; # Name the arguments.
   Mpp::log REP_LOAD => $dirinfo, $destdirinfo
     if $log_level;
+  undef $dirinfo->{xREPOSITORY};
 
   # TBD: Using a repository manifest (in addition to the possibility that
   # it's wrong) is still pretty slow. It would be better to do lazy computation
@@ -222,7 +226,7 @@ sub Mpp::Repository::get {
   # the linkee, which might be a local file, and which might have changed,
   # even if the link is still the same.
   my $linkee = readlink absolute_filename $src_finfo
-    if !defined $Mpp::symlink_in_rep_as_file && is_symbolic_link $src_finfo;
+    if !defined $Mpp::Repository::symlink_as_file && is_symbolic_link $src_finfo;
   my $binfo_finfo = $linkee ? file_info $linkee, $dest_finfo->{'..'} : $src_finfo;
   my $binfo = $binfo_finfo->{BUILD_INFO} ||=
     load_build_info_file( $binfo_finfo ) || {};
@@ -269,10 +273,9 @@ sub Mpp::Repository::get {
       ++$Mpp::failed_count;
       return 1;			# Indicate failure.
     }
-    ++$Mpp::rep_hits;
-  } else {
-    Mpp::log 'REP_EXISTING'
-      if $Mpp::log_level;
+    ++$Mpp::Repository::hits;
+  } elsif( Mpp::DEBUG ) {
+    Mpp::log 'REP_EXISTING';
   }
 
   # NOTE: This has to happen *after* the file exists (or else the build info
